"""Add AgentConfig table for custom agent management

Revision ID: 310819639420
Revises: 04ea79f25b7a
Create Date: 2025-10-28 09:10:18.321490

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
from airunner.utils.db import add_table, drop_table
from airunner.components.agents.data.agent_config import AgentConfig


# revision identifiers, used by Alembic.
revision: str = "310819639420"
down_revision: Union[str, None] = "04ea79f25b7a"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    add_table(AgentConfig)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    drop_table(AgentConfig)
    # ### end Alembic commands ###
