from sqlalchemy import Column, Integer, Boolean, String

from airunner.components.data.models.base import BaseModel
from airunner.enums import ModelService


class RAGSettings(BaseModel):
    __tablename__ = "rag_settings"
    id = Column(Integer, primary_key=True, autoincrement=True)
    model_service = Column(String, default=ModelService.LOCAL.value)
    model_path = Column(String, default="")
    chunk_size = Column(Integer, default=512)
    chunk_overlap = Column(Integer, default=50)
