WHISPER_FILES = {
    # faster-whisper CTranslate2 distil-large-v3 model
    # High quality, English-only, ~1.5GB disk, ~1.5-2GB VRAM with int8
    "Systran/faster-distil-whisper-large-v3": [
        "config.json",
        "model.bin",
        "preprocessor_config.json",
        "tokenizer.json",
        "vocabulary.json",
    ]
}
