"""rename crops_coord_top_left to crops_coords_top_left

Revision ID: 647a8e1098ce
Revises: e6ef79c17a07
Create Date: 2025-04-29 05:23:02.669468

"""

from typing import Sequence, Union

from airunner.components.art.data.generator_settings import GeneratorSettings
from airunner.utils.db import add_column, drop_column

# revision identifiers, used by Alembic.
revision: str = "647a8e1098ce"
down_revision: Union[str, None] = "e6ef79c17a07"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    add_column(
        GeneratorSettings,
        "crops_coords_top_left",
    )
    drop_column(GeneratorSettings, "crops_coord_top_left")


def downgrade() -> None:
    add_column(
        GeneratorSettings,
        "crops_coord_top_left",
    )
    drop_column(GeneratorSettings, "crops_coords_top_left")
