"""
Document editor component.

This component provides workspace management and file operations
for code editing and document management.
"""

from airunner.components.document_editor.workspace_manager import (
    WorkspaceManager,
)

__all__ = [
    "WorkspaceManager",
]
