"""Privacy settings widget module."""

from airunner.components.settings.gui.widgets.privacy_settings.privacy_settings_widget import (
    PrivacySettingsWidget,
)

__all__ = ["PrivacySettingsWidget"]
