"""
Fara Settings Package.

GUI components for configuring Fara-7B computer use agent.
"""

from airunner.components.settings.gui.widgets.fara_settings.fara_settings_widget import (
    FaraSettingsWidget,
)

__all__ = ["FaraSettingsWidget"]
