"""Knowledge data - now using daily markdown files.

The old KnowledgeFact and KnowledgeRelationship database models
have been removed in favor of daily markdown files stored in:
  ~/.local/share/airunner/text/knowledge/YYYY-MM-DD.md

See airunner.components.knowledge.knowledge_base for the new system.
"""

__all__ = []
