"""add_knowledge_migrated_flag

Revision ID: 500b6c395d38
Revises: 9623fe2fc53a
Create Date: 2025-10-28 12:38:02.780687

"""

from typing import Sequence, Union

from alembic import op
from airunner.utils.db import add_column
from airunner.components.settings.data.application_settings import (
    ApplicationSettings,
)


# revision identifiers, used by Alembic.
revision: str = "500b6c395d38"
down_revision: Union[str, None] = "9623fe2fc53a"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    add_column(ApplicationSettings, "knowledge_migrated")
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("application_settings", "knowledge_migrated")
    # ### end Alembic commands ###
