from airunner.components.downloader.gui.windows.installer.templates.completion_page_ui import (
    Ui_completion_page,
)
from airunner.components.downloader.gui.windows.setup_wizard.base_wizard import BaseWizard


class CompletionPage(BaseWizard):
    class_name_ = Ui_completion_page
